<?php
/**
 * Get block list of one region.
 *
 * @param  string    $module
 * @param  string    $method
 * @access public
 * @return array
 */
public function getPageBlocks($module, $method, $object = '')
{
    $device   = $this->app->clientDevice;
    $template = $this->config->template->{$device}->name;
    $theme    = $this->config->template->{$device}->theme;
    $pages    = "all,{$module}_{$method}";

    $layoutsInCurrent = array();
    if($object)
    {
        $layoutsInCurrent = $this->dao->select('*')->from(TABLE_LAYOUT)
            ->where('page')->eq("{$module}_{$method}")
            ->andWhere('template')->eq(!empty($this->config->template->{$device}->name) ? $this->config->template->{$device}->name : 'default')
            ->beginIF(version_compare($this->config->global->version, '1.4', '>='))->andWhere('theme')->in("all,$theme")->fi()
            ->beginIF(version_compare($this->config->global->version, '1.4', '<'))->andWhere('plan')->in("all,$theme")->fi()
            ->andWhere('object')->eq($object)
            ->fetchAll('region');
    }

    $rawLayouts = $this->dao->select('*')->from(TABLE_LAYOUT)
        ->where('page')->in($pages)
        ->andWhere('template')->eq(!empty($this->config->template->{$device}->name) ? $this->config->template->{$device}->name : 'default')
        ->beginIF(version_compare($this->config->global->version, '1.4', '>='))->andWhere('theme')->eq($theme)->fi()
        ->beginIF(version_compare($this->config->global->version, '1.4', '<'))->andWhere('plan')->eq($theme)->fi()
        ->andWhere('object')->eq('')
        ->fetchGroup('page', 'region');

    if(!empty($layoutsInCurrent))
    {
        foreach($layoutsInCurrent as $region => $layouts)
        {
            $rawLayouts["{$module}_{$method}"][$region] = $layouts;
        }
    }

    $blockIdList = array();
    foreach($rawLayouts as $page => $pageBlocks)
    {
        foreach($pageBlocks as $region => $regionBlock)
        {
            $regionBlocks = json_decode($regionBlock->blocks);
            foreach((array)$regionBlocks as $block)
            {
                $blockIdList[] = $block->id;
                if(isset($block->children)) foreach($block->children as $child) $blockIdList[] = $child->id;
            }
        }
    }

    $blocks = $this->dao->select('*')->from(TABLE_BLOCK)->where('id')->in($blockIdList)->fetchAll('id');

    $layouts = array();
    foreach($rawLayouts as $page => $pageBlocks)
    {
        $layouts[$page] = array();
        foreach($pageBlocks as $region => $regionBlock)
        {
            $layouts[$page][$region] = array();

            $regionBlocks = json_decode($regionBlock->blocks);

            foreach((array)$regionBlocks as $block)
            {
                if(!empty($blocks[$block->id]))
                {
                    $block->title    = $blocks[$block->id]->title;
                    $block->type     = $blocks[$block->id]->type;
                    $block->content  = $blocks[$block->id]->content;
                    $block->template = $blocks[$block->id]->template;
                }
                else
                {
                    $block->title    = '';
                    $block->type     = '';
                    $block->content  = '';
                    $block->template = '';
                }

                $mergedBlock = new stdclass();

                if(!empty($block->children))
                {
                    $mergedBlock->id = $block->id;
                    $children = array();
                    foreach($block->children as $child)
                    {
                        if(!empty($blocks[$child->id]))
                        {
                            $child->title    = $blocks[$child->id]->title;
                            $child->type     = $blocks[$child->id]->type;
                            $child->content  = $blocks[$child->id]->content;
                            $child->template = $blocks[$child->id]->template;
                        }
                        else
                        {
                            $child->title    = '';
                            $child->type     = '';
                            $child->content  = '';
                            $child->template = '';
                        }

                        $mergedChild = new stdclass();
                        $mergedChild->id          = zget($child, 'id', '');
                        $mergedChild->title       = zget($child, 'title', '');
                        $mergedChild->type        = zget($child, 'type', '');
                        $mergedChild->content     = zget($child, 'content', '');
                        $mergedChild->template    = zget($child, 'template', '');
                        $mergedChild->grid        = zget($child, 'grid', '');
                        $mergedChild->probability = zget($child, 'probability', '');
                        $mergedChild->isRandom    = zget($child, 'isRandom', '');
                        $mergedChild->titleless   = zget($child, 'titleless', '');
                        $mergedChild->borderless  = zget($child, 'borderless', '');
                        $children[] = $mergedChild;
                    }
                    $mergedBlock->children = $children;
                }
                else
                {
                    $mergedBlock->id       = $block->id;
                    $mergedBlock->title    = $block->title;
                    $mergedBlock->type     = $block->type;
                    $mergedBlock->content  = $block->content;
                    $mergedBlock->template = $block->template;
                }

                if(isset($block->grid))        $mergedBlock->grid        = $block->grid;
                if(isset($block->probability)) $mergedBlock->probability = $block->probability;
                if(isset($block->isRandom))    $mergedBlock->isRandom    = $block->isRandom;
                if(isset($block->titleless))   $mergedBlock->titleless   = $block->titleless;
                if(isset($block->borderless))  $mergedBlock->borderless  = $block->borderless;
                $layouts[$page][$region][] = $mergedBlock;
            }
        }
    }
    return $layouts;
}
